import sys
from flask import abort
import pymysql as mysql
from config import OPENAPI_AUTOGEN_DIR, DB_HOST, DB_USER, DB_PASSWD, DB_NAME

sys.path.append(OPENAPI_AUTOGEN_DIR)
from openapi_server import models


def db_cursor():
    return mysql.connect(host=DB_HOST, user=DB_USER, passwd=DB_PASSWD, db=DB_NAME).cursor()


def get_pokemons():
    with db_cursor() as cs:
        cs.execute("""
            SELECT p.id, p.name, poke.primary_type, poke.secondary_type, poke.generation, p.stamina, p.maxHP, p.defense,
            p.attack, p.height, p.weight, p.capture_rate, p.escape_rate, p.maxCP, CAST(poke.legendary='True' AS unsigned), press.image, h.Link 
            FROM pogo as p 
            INNER JOIN Pokemon as poke ON p.id = poke.id 
            INNER JOIN pokepress as press ON p.name = press.name
            INNER JOIN high_quality_image as h ON p.name = h.Name
            ORDER BY p.id
            """)
        result = [models.Pokemon(*pokemon) for pokemon in cs.fetchall()]
        return result


def get_pokemon(pokemonId):
    with db_cursor() as cs:
        cs.execute("""
            SELECT p.id, p.name, poke.primary_type, poke.secondary_type, poke.generation, p.stamina, p.maxHP, p.defense,
            p.attack, p.height, p.weight, p.capture_rate, p.escape_rate, p.maxCP, CAST(poke.legendary='True' AS unsigned), press.image, h.Link
            FROM pogo as p 
            INNER JOIN Pokemon as poke ON p.id = poke.id 
            INNER JOIN pokepress as press ON p.name = press.name
            INNER JOIN high_quality_image as h ON p.name = h.Name
            WHERE p.id=%s
            """, [pokemonId])
        result = cs.fetchone()
    if result:
        print(*result)
        return models.Pokemon(*result)
    else:
        abort(404)


def get_pokemon_by_type(pokemonType):
    with db_cursor() as cs:
        cs.execute(f"""
            SELECT p.id, p.name, poke.primary_type, poke.secondary_type, poke.generation, p.stamina, p.maxHP, p.defense,
            p.attack, p.height, p.weight, p.capture_rate, p.escape_rate, p.maxCP, CAST(poke.legendary='True' AS unsigned), press.image, h.Link
            FROM pogo as p 
            INNER JOIN Pokemon as poke ON p.id = poke.id 
            INNER JOIN pokepress as press ON p.name = press.name
            INNER JOIN high_quality_image as h ON p.name = h.Name
            WHERE poke.primary_type='{pokemonType}' OR poke.secondary_type='{pokemonType}'
            ORDER BY p.id
            """)
        return [models.Pokemon(*pokemon) for pokemon in cs.fetchall()]


def get_average_generation_stat():
    with db_cursor() as cs:
        cs.execute("""
            SELECT poke.generation, AVG(p.stamina), AVG(p.maxHP), AVG(p.defense), AVG(p.attack), AVG(p.height),
            AVG(p.weight), AVG(p.capture_rate), AVG(p.escape_rate), AVG(p.maxCP)
            FROM pogo as p 
            INNER JOIN Pokemon as poke ON p.id = poke.id 
            INNER JOIN pokepress as press ON p.name = press.name
            GROUP BY poke.generation
            """)

        return [models.StatGeneration(*result) for result in cs.fetchall()]


def get_pokemon_best_capture_by_type(pokemonType, amount):
    with db_cursor() as cs:
        cs.execute(f"""
            SELECT p.id, p.name, poke.primary_type, poke.secondary_type, poke.generation, p.stamina, p.maxHP, p.defense,
            p.attack, p.height, p.weight, p.capture_rate, p.escape_rate, p.maxCP, CAST(poke.legendary='True' AS unsigned), press.image, h.Link
            FROM pogo as p 
            INNER JOIN Pokemon as poke ON p.id = poke.id 
            INNER JOIN pokepress as press ON p.name = press.name
            INNER JOIN high_quality_image as h ON p.name = h.Name
            WHERE poke.primary_type='{pokemonType}' OR poke.secondary_type='{pokemonType}'
            ORDER BY CAST(p.capture_rate AS FLOAT) DESC
            """)
        result = [models.Pokemon(*pokemon) for pokemon in cs.fetchall()]
        try:
            return result[0:amount]
        except IndexError:
            return result


def get_pokemon_eggs():
    with db_cursor() as cs:
        cs.execute("""
          SELECT e.id, e.name, e.distance, e.min, e.max 
          FROM egg_hatching as e
          ORDER BY e.id
          """)
        result = [models.Egg(*egg) for egg in cs.fetchall()]
        return result


def get_pokemon_egg_by_id(pokemonId):
    with db_cursor() as cs:
        cs.execute(f"""
          SELECT e.id, e.name, e.distance, e.min, e.max 
          FROM egg_hatching as e
          WHERE e.id='{pokemonId}'
          ORDER BY e.id
          """)
        result = cs.fetchone()
    if result:
        return models.Egg(*result)
    else:
        abort(404)


def get_pokemon_eggs_by_distance(distance):
    with db_cursor() as cs:
        cs.execute("""
            SELECT e.id, e.name, e.distance, e.min, e.max 
            FROM egg_hatching as e
            WHERE e.distance=%s
            ORDER BY e.id
            """, [distance])
        return [models.Egg(*egg) for egg in cs.fetchall()]


def get_pokemon_eggs_by_type(type):
    with db_cursor() as cs:
        cs.execute(f"""
            SELECT e.id, e.name, e.distance, e.min, e.max 
            FROM egg_hatching as e
            INNER JOIN Pokemon as p 
            ON p.id = e.id
            WHERE p.primary_type='{type}' OR p.secondary_type='{type}'
            ORDER BY e.id
            """)
        return [models.Egg(*egg) for egg in cs.fetchall()]


def get_pokemon_eggs_by_gen(generation):
    with db_cursor() as cs:
        cs.execute(f"""
            SELECT e.id, e.name, e.distance, e.min, e.max 
            FROM egg_hatching as e
            INNER JOIN Pokemon as p 
            ON p.id = e.id
            WHERE p.generation={generation}
            ORDER BY e.id
            """)
    return [models.Egg(*egg) for egg in cs.fetchall()]


def get_ranking_pokemon_that_have_egg_by_capture_rate():
    with db_cursor() as cs:
        cs.execute(f"""
            SELECT p.id, p.name, poke.primary_type, poke.secondary_type, poke.generation, p.stamina, p.maxHP, p.defense,
            p.attack, p.height, p.weight, p.capture_rate, p.escape_rate, p.maxCP, 
            CAST(poke.legendary='True' AS unsigned), press.image, h.Link
            FROM egg_hatching as e 
            INNER JOIN pogo as p ON p.id = e.id
            INNER JOIN Pokemon as poke ON p.id = poke.id 
            INNER JOIN pokepress as press ON p.name = press.name
            INNER JOIN high_quality_image as h ON p.name = h.Name
            ORDER BY (p.capture_rate*p.maxCP)
            DESC
            """)
    return [models.Pokemon(*pokemon) for pokemon in cs.fetchall()]


def get_damage_to_type(pokemonId):
    """
        How to calculate damage for pokemon with 2 type?
        Ex Name         | PriType | SecType | Attack
           Bulbasaur    | Grass   | Poison  | 49

           Type         | Grass    |
           Grass        | 0.625    |
           Poison       | 1.6      |

           Bulbasaur damage to fire pokemon is 0.625*1.6*49 =  49
    """
    with db_cursor() as cs:
        cs.execute(f"""
            SELECT stat.id, stat.name, MAX(stat.attack) * EXP(SUM(LOG(stat.normal))) as normal,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.fire))) as fire, 
                    MAX(stat.attack) * EXP(SUM(LOG(stat.water))) as water, 
                    MAX(stat.attack) * EXP(SUM(LOG(stat.electric))) as electric,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.grass))) as grass, 
                    MAX(stat.attack) * EXP(SUM(LOG(stat.ice))) as ice,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.fighting))) as fighting, 
                    MAX(stat.attack) * EXP(SUM(LOG(stat.poison))) as poison,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.ground))) as ground,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.flying))) as flying,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.psychic))) as psychic,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.bug))) as bug,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.rock))) as rock,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.ghost))) as ghost,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.dragon))) as dragon,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.dark))) as dark,
                    MAX(stat.attack) * EXP(SUM(LOG(stat.steel))) as steel, 
                    MAX(stat.attack) * EXP(SUM(LOG(stat.fairy))) as fairy
                    FROM (
                        SELECT pg.id, pg.name, pg.attack, p.primary_type, p.secondary_type, 
                                t.normal, t.fire, t.water, t.electric, t.grass, t.ice, 
                                t.fighting, t.poison, t.ground, t.flying, t.psychic, 
                                t.bug, t.rock, t.ghost, t.dragon, t.dark, t.steel, t.fairy
                        FROM pokemon as p
                        INNER JOIN pogo as pg ON p.id = pg.id
                        INNER JOIN type_chart as t ON p.primary_type = t.type
                        WHERE p.id = {pokemonId}
                        union
                        SELECT pg.id, pg.name, pg.attack, p.primary_type, p.secondary_type, 
                                t.normal, t.fire, t.water, t.electric, t.grass, t.ice, 
                                t.fighting, t.poison, t.ground, t.flying, t.psychic, 
                                t.bug, t.rock, t.ghost, t.dragon, t.dark, t.steel, t.fairy                        
                        FROM pokemon as p
                        INNER JOIN pogo as pg ON p.id = pg.id
                        INNER JOIN type_chart as t ON p.secondary_type = t.type
                        WHERE p.id = {pokemonId}
                    ) stat
                    group by stat.id, stat.name
            """)
        result = cs.fetchone()
        if result:
            return models.DamageType(*result)
        else:
            abort(404)
