DROP DATABASE IF EXISTS pokemongo;
CREATE DATABASE IF NOT EXISTS pokemongo;
USE pokemongo;

CREATE TABLE IF NOT EXISTS pogo (
    name VARCHAR(45),
    id INT NOT NULL,
    stamina INT,
    attack INT,
    defense INT,
    maxHP INT,
    capture_rate FLOAT,
    escape_rate FLOAT,
    weight FLOAT,
    height FLOAT,
    maxCP INT,
    PRIMARY KEY (id)
);

LOAD DATA LOCAL INFILE './data/csv/pogo.csv'
INTO TABLE pogo
FIELDS TERMINATED BY ','
LINES TERMINATED BY '\n'
IGNORE 1 ROWS;

UPDATE pogo
SET name = 'Mime Jr.'
WHERE name='Mime';

UPDATE pogo
SET name = 'Mr. Mime'
WHERE name='Mr.';

CREATE TABLE IF NOT EXISTS high_quality_image (
    ID INT NOT NULL,
    Name VARCHAR(45),
    Form VARCHAR(45),
    Link VARCHAR(300)
);

LOAD DATA LOCAL INFILE './data/csv/high_quality_image.csv'
INTO TABLE high_quality_image
FIELDS TERMINATED BY ','
LINES TERMINATED BY '\n'
IGNORE 1 ROWS;

UPDATE high_quality_image
SET Form = 'Standard'
WHERE Form='';

UPDATE high_quality_image
SET name = CONCAT(Name, ' ', Form)
WHERE Form<>'Standard';


DROP TABLE IF EXISTS egg_hatching;
CREATE TABLE IF NOT EXISTS egg_hatching (
    id INT NOT NULL,
    distance VARCHAR(10),
    name VARCHAR(45),
    min INT,
    max INT,
    PRIMARY KEY (id)
);

LOAD DATA LOCAL INFILE './data/csv/egg_hatching.csv'
INTO TABLE egg_hatching
FIELDS TERMINATED BY ','
LINES TERMINATED BY '\n'
IGNORE 1 ROWS;


DROP TABLE IF EXISTS Pokemon;
CREATE TABLE IF NOT EXISTS Pokemon (
    id INT NOT NULL,
    name VARCHAR(45),
    primary_type VARCHAR(45),
    secondary_type VARCHAR(45) DEFAULT NULL,
    total INT,
    hp INT,
    attack INT,
    defense INT,
    spAtk INT,
    spDef INT,
    speed INT,
    generation INT,
    legendary VARCHAR(5),
    PRIMARY KEY (id)
);

LOAD DATA LOCAL INFILE './data/csv/Pokemon.csv'
INTO TABLE Pokemon
FIELDS TERMINATED BY ','
LINES TERMINATED BY '\n'
IGNORE 1 ROWS;

UPDATE Pokemon
SET secondary_type = null
WHERE secondary_type='';


DROP TABLE IF EXISTS pokepress;
CREATE TABLE IF NOT EXISTS pokepress (
    id INT NOT NULL,
    name VARCHAR(45),
    candy VARCHAR(45),
    charge2 VARCHAR(45) DEFAULT NULL,
    form VARCHAR(45),
    image VARCHAR(300),
    PRIMARY KEY (name)
);

LOAD DATA LOCAL INFILE './data/csv/pokemon_images.csv'
INTO TABLE pokepress
FIELDS TERMINATED BY ','
LINES TERMINATED BY '\n'
IGNORE 1 ROWS;

UPDATE pokepress
SET charge2 = null
WHERE charge2='';

DROP TABLE IF EXISTS type_chart;
CREATE TABLE IF NOT EXISTS type_chart (
    type VARCHAR(45),
    Normal FLOAT,
    Fire FLOAT,
    Water FLOAT,
    Electric FLOAT,
    Grass FLOAT,
    Ice FLOAT,
    Fighting FLOAT,
    Poison FLOAT,
    Ground FLOAT,
    Flying FLOAT,
    Psychic FLOAT,
    Bug FLOAT,
    Rock FLOAT,
    Ghost FLOAT,
    Dragon FLOAT,
    Dark FLOAT,
    Steel FLOAT,
    Fairy FLOAT,
    PRIMARY KEY (type)
);

LOAD DATA LOCAL INFILE './data/csv/Pokemon_Type_Chart.csv'
INTO TABLE type_chart
FIELDS TERMINATED BY ','
LINES TERMINATED BY '\n'
IGNORE 1 ROWS;