import csv
import requests
from bs4 import BeautifulSoup, ResultSet


def get_all_filed() -> list:
    res = requests.get("https://gamepress.gg/pokemongo/pokemon-go-eggs-list")
    soup = BeautifulSoup(res.content, 'html.parser')
    pokemon_list_section = soup.find("div", {"class": "field field--name-field-content-to-embed2 field--type-entity-"
                                            "reference-revisions field--label-hidden field__items"})
    field_items = pokemon_list_section.find_all("div", {"class": "field__item"})
    kilometer = ""
    all_pokemons = []

    for each_field in field_items:
        if kilometer != distance_value(each_field, kilometer):
            kilometer = distance_value(each_field, kilometer)

        all_pokemons_by_genaration = get_all_pokemon_detail(each_field, kilometer)
        if all_pokemons_by_genaration:
            for pokemon in all_pokemons_by_genaration:
                all_pokemons.append(pokemon)
    return all_pokemons


def distance_value(field: BeautifulSoup, default_value: str) -> list:
    def has_id(tag):
        return tag.has_attr("id")

    if field.find(has_id):
        id_text = field.find(has_id).get('id')
        if 'km' in id_text:
            return id_text
    return default_value


def get_all_pokemon_detail(field: BeautifulSoup, kilometer: str) -> list:
    all_eggs_stat_with_tag = field.find_all("div", {"class": "views-col-egg", "class": "clearfix"})
    eggs_stat = get_pokemon_egg_stat(all_eggs_stat_with_tag)
    poke_name = [pokemon_with_tag.string for pokemon_with_tag in field.find_all("a", {"class": "pokemon-title"})]
    poke_id = get_pokemon_id(field)

    if len(eggs_stat) == len(poke_name):
        all_detail = []
        for index in range(len(poke_name)):
            poke_info = [poke_id[index], kilometer, poke_name[index].string, eggs_stat[index][0], eggs_stat[index][1]]
            all_detail.append(poke_info)
        return all_detail
    return None


def get_pokemon_id(field: BeautifulSoup) -> list:
    poke_id_with_tag = [pokemon_with_tag for pokemon_with_tag in field.find_all("small")]
    poke_id = []
    for each in poke_id_with_tag:
        poke_id.append(each.prettify().split()[1].replace('#', ""))
    return poke_id


def get_pokemon_egg_stat(field: ResultSet):
    all_stat = []
    for element in field:
        min_and_max_egg_stat = element.find_all("div", {"class": "cp-egg"})
        stat = []
        for each_stat in min_and_max_egg_stat:
            stat.append(each_stat.prettify().split()[-2])
        all_stat.append(stat)
    return all_stat


index = 0
all_poke_data = get_all_filed()
with open('./data/csv/egg_hatching.csv', 'w', newline='') as file:
    writer = csv.writer(file)
    writer.writerow(["ID", "Distance", "Name", "Min", "Max"])
    for each_poke_data in all_poke_data:
        writer.writerow(each_poke_data)
