import csv
import requests
from bs4 import BeautifulSoup


def get_all_filed() -> list:
    res = requests.get("https://gamepress.gg/pokemongo/pokemon-list")
    soup = BeautifulSoup(res.content, 'html.parser')
    pokemon_list_section = soup.find("tbody", {"id": "pokemon-list"})
    field_items = pokemon_list_section.find_all("tr", {"class": "pokemon-row"})

    pokemon_list = []

    for each_field in field_items:
        pokemon_id = each_field["data-id"]

        # Fixed Rapidash has no ID
        if not pokemon_id:
            pokemon_id = 78

        pokemon_name = each_field["data-name"]
        pokemon_candy = each_field["data-candy"]
        pokemon_charge2 = each_field["data-charge2"]

        pokemon_form = get_poke_form(pokemon_name)

        pokemon_image = "https://gamepress.gg" + each_field.select_one("td:nth-of-type(2) a:nth-of-type(1) img")["src"]
        pokemon_list.append([pokemon_id, pokemon_name, pokemon_candy, pokemon_charge2, pokemon_form, pokemon_image])
    return pokemon_list


def get_poke_form(name: str) -> str:
    split_text = name.split(" ")
    if len(split_text) == 1:
        form = "Standard"

    elif len(split_text) == 2:
        if split_text[1].startswith("("):
            form = split_text[1]
            form = form.replace("(", "")
            form = form.replace(")", "")
        else:
            form = split_text[0]

    elif len(split_text) == 4:
        if split_text[0] == "Aegislash":
            form = split_text[2]
        else:
            form = f"{split_text[0]} {split_text[2]} {split_text[3]}"

    elif len(split_text) == 5:
        form = f"{split_text[0]} {split_text[3]} {split_text[4]}"

    else:
        if split_text[1].startswith("("):
            form = f"{split_text[1].replace('(', '')} {split_text[2].replace(')', '')}"
        elif split_text[0] == "Aegislash":
            form = split_text[-1]
        else:
            form = f"{split_text[0]} {split_text[-1]}"
    return form


with open('./data/csv/pokemon_images.csv', 'w', newline='') as file:
    writer = csv.writer(file)
    pokemon_list = get_all_filed()
    pokemon_list.sort(key=lambda x: int(x[0]))
    writer.writerow(["ID", "Name", "Candy", "Charge2", "Form", "Image"])
    for each_poke_data in pokemon_list:
        writer.writerow(each_poke_data)
